/*
Copyright (c) 2004 Dmitry Markman, PhD dmarkman@mac.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. The name of the author may not be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package org.dm.webkit.dom;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.DOMException;
import org.w3c.dom.Text;

import org.dm.webkit.DMWebKitView;

public class TextWrapper extends NodeWrapper implements Text{
String data;
int length;
    public TextWrapper(DMWebKitView owner,String nodeAccessor,DOMDocument ownerDocument,Node parentNode){
        super(owner,nodeAccessor,TEXT_NODE,ownerDocument,parentNode);
        defineData();
    }

    protected void defineData(){
        if(isOrphan()) return;
        data = JSAccessors.getProperty(owner,nodeAccessor,"data");
        String str = JSAccessors.getProperty(owner,nodeAccessor,"length");
        try{
            length = Integer.parseInt(str);
        }catch(Throwable t){
            length = 0;
        }
    }

    public void setNodeValue(String nodeValue) throws DOMException{
        if(isOrphan() && nodeValue != null){
            data = nodeValue;
            length = data.length();
        }else{
            super.setNodeValue(nodeValue);
            defineData();
        }
    }

    public String getCreateString(){
        if(!isOrphan()) return super.getCreateString();
        StringBuffer sb = new StringBuffer();
        sb.append("var newNode = window.document.createTextNode(\"");
        sb.append(data);
        sb.append("\");\n");
        return sb.toString();
    }

    public String toString(){
        return "TEXT [data "+getData().trim()+" length "+getLength()+"]"+"    ["+getNodeType()+"]";
    }
    public String getData(){return (data != null)?data:"";}
    public int getLength(){return (data != null)?length:0;}
    
    public Text splitText(int offset) throws DOMException{
        int myIndex = -1;
        if(parentNode instanceof NodeWrapper){
            myIndex = ((NodeWrapper)parentNode).getChildIndex(this);
        }
        JSAccessors.callMethod(owner,nodeAccessor,"splitText",new String[]{""+offset});
        if(parentNode instanceof NodeWrapper){
            ((NodeWrapper)parentNode).redefineChildNodes();
        }
        if(myIndex != -1){
            NodeList children = parentNode.getChildNodes();
            Node newNode = children.item(myIndex);
            if(newNode instanceof TextWrapper){
                return (TextWrapper)newNode;
            }
        }
        return null;
    }
    
    public void setData(String arg) throws DOMException{
        JSAccessors.setProperty(owner,nodeAccessor,"data",arg);
        normalize();
        defineData();
    }
    
    public void appendData(String arg) throws DOMException{
        JSAccessors.callMethod(owner,nodeAccessor,"appendData",new String[]{arg});
        normalize();
        defineData();
    }
    
    public void replaceData(int offset, int count, String arg) throws DOMException{
        JSAccessors.callMethod(owner,nodeAccessor,"replaceData",new String[]{""+offset,""+count,arg});
        defineData();
    }
    
    public void deleteData(int offset, int count) throws DOMException{
        JSAccessors.callMethod(owner,nodeAccessor,"deleteData",new String[]{""+offset,""+count});
        defineData();
    }
    
    public void insertData(int offset, String arg) throws DOMException{
        JSAccessors.callMethod(owner,nodeAccessor,"insertData",new String[]{""+offset,arg});
        defineData();
    }
    
    public String substringData(int offset, int count) throws DOMException{
        return JSAccessors.callMethod(owner,nodeAccessor,"substringData",new String[]{""+offset,""+count});
    }
    
}

