/*
Copyright (c) 2004 Dmitry Markman, PhD dmarkman@mac.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. The name of the author may not be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package org.dm.webkit.installer;

import java.io.*;
import com.apple.eio.FileManager;

public class WebKitInstaller{
    private static final String JavaExtensions          = "Java/Extensions";
    private static final int    kDomainLibraryFolder    = 0x646C6962;//dlib /* the Library subfolder of a particular domain*/
    private static final short  kUserDomain             = -32763; /* Read/write. Resources that are private to the user.*/

    private static final String libName                 = "libwebkitsample.jnilib";
    private static final String resourcePath            = "/org/dm/webkit/installer/macosx/"+libName;


    File libFolder = null;
    File jnilibFile = null;

    public WebKitInstaller(){
        createLibFolder();
        jnilibFile = getJNILibFromBundle();
        if(jnilibFile == null){
            jnilibFile = new File(libFolder,libName);
            copyResourceToFile(resourcePath,jnilibFile);
        }
    }
    
    File getJNILibFromBundle(){
        File retValue = null;
        try{
            com.apple.cocoa.foundation.NSBundle mainBundle = com.apple.cocoa.foundation.NSBundle.mainBundle();
            retValue = new File(mainBundle.bundlePath()+"/Contents/Resources/Java/"+libName);
            if(retValue.exists()) return retValue;
        }catch(Throwable t){
            retValue = null;
        }
        return null;
    }
    
    public String getJNILibLibraryPath(){
        try{
            return jnilibFile.getCanonicalPath();
        }catch(Throwable t){}
        return null;
    }
    
    protected void createLibFolder(){
        String folderName = getFolderPath(kUserDomain,kDomainLibraryFolder);
        if(folderName == null) return;
        File libFile = new File(folderName);
        libFolder = new File(libFile,JavaExtensions);
        if(!libFolder.exists()){
            boolean success = libFolder.mkdirs();
        }
        if(!libFolder.exists()){
            libFolder = null;
        }
    }

    public static void copyResourceToFile(String absResPath,java.io.File file){
        try{
            InputStream is = WebKitInstaller.class.getResourceAsStream(absResPath);
            byte []buffer = new byte[4096];
            int rb = 0;
            if(is != null){
                FileOutputStream fos = new FileOutputStream(file);
                while((rb = is.read(buffer,0,buffer.length)) > 0){
                    fos.write(buffer,0,rb);
                }
                fos.close();
            }
        }catch(Throwable t){
            t.printStackTrace();
        }
    }
    
    public static String  getFolderPath(short domain,int folderKind){
        try{
            return FileManager.findFolder(domain,folderKind);
        }catch(Throwable t){}
        return null;
    }

}


